<?php

namespace Teknisa\FIN\Modules\ContaPagar\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Model\Financeiro\Entities\Cnabpag;
use Teknisa\Model\Financeiro\Entities\Logboletorempag;

/**
 * Class RemessaTituloFactory
 * @package Teknisa\FIN\Modules\ContaPagar\Factory
 */
class RemessaTituloFactory {

    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    /**
     * Busca CNAB Pagar
     * @param $nrorg
     * @param $cdempresa
     * @param $nrcnabpag
     * @return Cnabpag
     */
    public function buscaCnabpag($nrorg, $cdempresa, $nrcnabpag){
        /** @var $params */
        $params = array();
        $params['nrorg']     = $nrorg;
        $params['cdempresa'] = $cdempresa;
        $params['nrcnabpag'] = $nrcnabpag;

        /** @var Cnabpag $cnabpag */
        $cnabpag = $this->entityManager->getRepository(Cnabpag::class)->findOneBy($params);

        return $cnabpag;
    }

    /**
     * Busca Boleto agar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @return Boletpagar
     */
    public function buscaBoletpagar($nrorg, $cdfilial, $nrlanbolpag){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanbolpag'] = $nrlanbolpag;

        /** @var Boletpagar $boletpagar */
        $boletpagar = $this->entityManager->getRepository(Boletpagar::class)->findOneBy($params);

        return $boletpagar;
    }

    /**
     * Busca Boleto Pagar Remessa
     * @param $nrorg
     * @param $cdempresa
     * @param $nrcnabpagrem
     * @return Boletpagar
     */
    public function buscaBoletpagarRem($nrorg, $cdempresa, $nrcnabpagrem){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdempresa']    = $cdempresa;
        $params['nrcnabpagrem'] = $nrcnabpagrem;
        
        /** @var Boletpagar $boletpagar */
        $boletpagar = $this->entityManager->getRepository(Boletpagar::class)->findBy($params);
        
        return $boletpagar;
    }

    /**
     * Busca Log Boleto Remessa Pagar
     * @param $nrorg
     * @param $cdfiliallogpag
     * @param $nrlanbolpaglog
     * @param $cdempresalogpag
     * @param $nrcnabpagretlog
     * @return Logboletorempag
     */
    public function buscaLogboletorempag($nrorg, $cdfiliallogpag, $nrlanbolpaglog, $cdempresalogpag, $nrcnabpagretlog){
        /** @var $params */
        $params = array();
        $params['nrorg']           = $nrorg;
        $params['cdfiliallogpag']  = $cdfiliallogpag;
        $params['nrlanbolpaglog']  = $nrlanbolpaglog;
        $params['cdempresalogpag'] = $cdempresalogpag;
        $params['nrcnabpagretlog'] = $nrcnabpagretlog;

        /** @var Logboletorempag $logboletorempag */
        $logboletorempag = $this->entityManager->getRepository(Logboletorempag::class)->findOneBy($params);

        return $logboletorempag;
    }

    /**
     * Insere CNAB Pagar
     * @param $nrorg
     * @param $cdempresa
     * @param $nrcnabpag
     * @param $cdoperador
     * @param $dtcnabpag
     * @param $idtipcnabpag
     * @param $cdreducont
     * @param $cdbanco
     * @param $idopecnabpag
     * @param $qtbolenvcnabpag
     * @param $vrtotbolcnabpag
     * @param $nrseqremessapagar
     * @param $dsarqcnabpag
     * @param $cdfilial
     * @param $idagenremcnabpag
     * @param $idtpremcnabpag
     * @return Cnabpag
     */
    public static function insereCnabPag(
        $nrorg, $cdempresa, $nrcnabpag, $cdoperador, $dtcnabpag, $idtipcnabpag,
        $cdreducont, $cdbanco, $idopecnabpag, $qtbolenvcnabpag,
        $vrtotbolcnabpag, $nrseqremessapagar, $dsarqcnabpag, $cdfilial, $idagenremcnabpag, $idtpremcnabpag,
        $idprenegociado
    ){
        /** @var Cnabpag $cnabpag */
        $cnabpag = New Cnabpag();
        $cnabpag->setNrorg($nrorg);
        $cnabpag->setCdempresa($cdempresa);
        $cnabpag->setNrcnabpag($nrcnabpag);
        $cnabpag->setCdoperador($cdoperador);
        $cnabpag->setDtcnabpag($dtcnabpag);
        $cnabpag->setIdtipcnabpag($idtipcnabpag);
        $cnabpag->setCdreducont($cdreducont);
        $cnabpag->setCdbanco($cdbanco);
        $cnabpag->setIdopecnabpag($idopecnabpag);
        $cnabpag->setQtbolcnabpag($qtbolenvcnabpag);
        $cnabpag->setVrtotalcnabpag($vrtotbolcnabpag);
        $cnabpag->setNrseqremessapag($nrseqremessapagar);
        $cnabpag->setDsarqcnabpag($dsarqcnabpag);
        $cnabpag->setCdfilial($cdfilial);
        $cnabpag->setIdagenremcnabpag($idagenremcnabpag);
        $cnabpag->setIdtpremcnabpag($idtpremcnabpag);
        $cnabpag->setIdprenegociado($idprenegociado);
        $cnabpag->setDtinclusao(DateUtil::getDataAtual());
        $cnabpag->setDtultatu(DateUtil::getDataAtual());

        return $cnabpag;
    }

    /**
     * Insere Log Boleto Remessa
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $nrcnabpag
     * @param $cdempresa
     * @param $cdoperador
     * @return Logboletorempag
     */
    public static function insereLogBoletoRem(
        $nrorg, $cdfilial, $nrlanbolpag, $nrcnabpag, $cdempresa, $cdoperador
    ){
        /** @var Logboletorempag $logboletorempagar */
        $logboletorempagar = New Logboletorempag();
        $logboletorempagar->setNrorg($nrorg);
        $logboletorempagar->setCdfiliallogpag($cdfilial);
        $logboletorempagar->setNrlanbolpaglog($nrlanbolpag);
        $logboletorempagar->setNrcnabpagretlog($nrcnabpag);
        $logboletorempagar->setCdempresalogpag($cdempresa);
        $logboletorempagar->setCdopelogpag($cdoperador);
        $logboletorempagar->setDtlogboletrpag(DateUtil::getDataAtual());
        $logboletorempagar->setDtinclusao(DateUtil::getDataAtual());

        return $logboletorempagar;
    }

    /**
     * Atualiza Boleto Pagar
     * @param Boletpagar $boletpagar
     * @param $nrcnabpag
     * @return Boletpagar
     */
    public static function atualizaBoletpagar(
        Boletpagar $boletpagar, $nrcnabpag, $idsitucnabp
    ){
        $boletpagar->setNrcnabpagrem($nrcnabpag);
        $boletpagar->setIdsitucnabp($idsitucnabp);

        return $boletpagar;
    }

    /**
     * Atualiza Cnab Pagar
     * @param Cnabpag $cnabpag
     * @param $dsarqcnabpag
     * @return Cnabpag
     */
    public static function atualizaCnabpag(
        Cnabpag $cnabpag, $dsarqcnabpag
    ){
        $cnabpag->setDsarqcnabpag($dsarqcnabpag);

        return $cnabpag;
    }

}