<?php
namespace Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco;

use Fiscal\Entities\Boletpagar;
use Financeiro\Util\Util;
use Teknisa\Model\Financeiro\Entities\Cnabpag;
use Fiscal\Repositories\Contapagar as rep_Contapagar;
use Geral\Entities\Empresa;
use Geral\Entities\Filial;
use Geral\Entities\Contcorr;
use Geral\Entities\Tipobaixapag;
use Teknisa\Libs\Util\DateUtil;
use Fiscal\Entities\Contapagar;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\GeraCnabService;


/**
 * Class GeraCnabBradescoService
 * @package Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco
 */
class GeraCnabBradescoService extends GeraCnabService{

    CONST TIPO_INS_JUR_CPF = 1;
    CONST TIPO_INS_JUR_CNPJ = 2;
    CONST TIPO_INS_JUR_OUTROS = 3;
    CONST PAGTO_FORNECEDORES = '20';
    CONST ORIGEM_CLIENTE = '1';
    CONST ID_REGISTRO_HEADER = '0';
    CONST ID_REGISTRO_TRANSACAO = '1';
    CONST ID_REGISTRO_TRAILLER = '9';

    CONST MODALIDADE_BOLETO_BANCARIO = '31';
    CONST MODALIDADE_ORDEM_PAGAMENTO = '02';
    CONST MODALIDADE_CREDITO_EM_CONTA = '01';
    CONST MODALIDADE_TED_CIP = '07';
    CONST MODALIDADE_TED_STR = '08';
    CONST MODALIDADE_DOC = '03';

    //Número sequencial do registro;
    private $nrseqreg = 0;
    //Somatório dos valores de pagamentos
    private $valorTotalPgto = 0;

    /**
     * @param Cnabpag $remessa
     * @throws \Exception
     */
    public function __construct(Cnabpag $remessa){

        try {
            $this->nrorg = $remessa->getNrorg();

            $this->remessa = $remessa;

            // Carrega os atributos Filial e Empresa
            $this->filial = Filial::findOneByCdfilialAndNrorg($remessa->getCdfilial(), $remessa->getNrorg());
            $this->empresa = Empresa::findOneByCdempresaAndNrorg($remessa->getCdempresa(), $remessa->getNrorg());

            // Busca dados referente a filial se foi informada
            if ($remessa->getCdfilial()) {
                if ($this->filial->getIdtpijurfili() == Filial::TIPO_INSCRICAO_FISICA){
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                } elseif($this->filial->getIdtpijurfili() == Filial::TIPO_INSCRICAO_CGC){
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;
                }else{
                    $this->tipoinscricao = self::TIPO_INS_JUR_OUTROS;
                }

                $this->inscricaojuridica = Util::removeMascaraInscricao($this->filial->getNrinsjurfili());
                $this->nomeempresa = $this->filial->getNmrazsocfili();
            } // Se nao busca dados da Empresa
            else {
                if ($this->empresa->getIdtpijurempr() == Empresa::TIPO_INSCRICAO_FISICA){
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                }elseif($this->empresa->getIdtpijurempr() == Empresa::TIPO_INSCRICAO_CGC){
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;
                }else{
                    $this->tipoinscricao = self::TIPO_INS_JUR_OUTROS;
                }

                $this->inscricaojuridica = Util::removeMascaraInscricao($this->empresa->getNrinsjurempr());
                $this->nomeempresa = $this->empresa->getNmrazsocempr();
            }

            $this->contcorr = Contcorr::findOneByCdreducontAndNrorg($remessa->getCdreducont(), $remessa->getNrorg());
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @return string
     * @throws \Exception
     */
    public function geraHeaderArquivo()
    {
        try {// CAMPO: Identificação do registro
            // POSIÇÃO: 1 a 1
            // PICTURE: 9(1)
            // CONTEÚDO: Obrigatório - fixo "zero (0)
            $linha = self::ID_REGISTRO_HEADER;

            // Cód. de comunicação = *identificação da empresa no banco. Fornecido pelo banco*
            // Posição: 2 a 9
            //Picture: 9(8)
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(Util::campoCnabValido(substr(trim($identpag),0,8)),8,'0',STR_PAD_LEFT);
            }else{
                $linha.= str_repeat('0',8);
            }

            //Tipo inscrição da empresa (1=CPF/2=CGC/3=Outros).
            //Posição: 10 a 10 - Picture: 9(1)
            $linha.= $this->tipoinscricao;

            //CGC empresa pagadora + CGC <-> Filial + CGC/CPF <-> Controle.
            //(15 posições numericas)
            $linha.= str_pad(trim($this->inscricaojuridica),15,'0',STR_PAD_LEFT);

            //Nome da empresa pagadora - Posição: 26 a 65 - Picture: X(40)
            $linha.= str_pad(substr(trim(Util::campoCnabValido($this->nomeempresa)),0,40),40);

            //Tipo de serviço (pagto fornecedores) - Posição: 66 a 67 - Picture: 9(2)
            $linha.= self::PAGTO_FORNECEDORES;

            //Código de origem do arquivo (1 = origem do cliente, 2 = origem no banco).
            //Obrigatório Fixo = "1' - Posição: 68 a 68 - Picture: 9(1)
            $linha.= self::ORIGEM_CLIENTE;

            //Nº da Remessa - Posição: 69 a 73 - Picture: 9(5)
            $linha.= substr($this->remessa->getNrseqremessapag(),-5);

            //Nº do Retorno (Desconsiderado - fixos zeros - (Valido somente p/ o arq. retorno)).
            //Posição: 74 a 78 - Picture: 9(5)
            $linha.= str_repeat('0',5);

            //Data da gravação do arquivo - Posição: 79 a 86 - Picture: 9(8)
            $data = DateUtil::getDataAtual();
            $linha.= $data->format('Ymd');

            //Hora de gravação do arquivo - Posição: 87 a 92 - Picture 9(6)
            $linha.= $data->format('His');

            //Densidade de gravação do arq./Fita (Brancos) - Posição: 93 a 97 - Picture: X(5)
            $linha.= str_repeat(' ',5);

            //Unidade de Densidade de gravação/Fita - Posição: 98 a 100 - Picture: X(3)
            $linha.= str_repeat(' ', 3);

            //Identificação módulo micro (Brancos) - Posição: 101 a 105 - Picture: X(5)
            $linha.= str_repeat(' ', 5);

            //Tipo do processamento (DESCONSIDERADO - campo válido somente p/ o arq. RETORNO)
            //Posição: 106 a 106 - Picture: 9(1)
            $linha.= '0';

            //Reservado empresa (para uso da empresa - Posição: 107 a 180 - Picture: X(74)
            $linha.= str_repeat(' ', 74);

            //Reservado banco - Posição: 181 a 260 - Picture: X()
            $linha.= str_repeat(' ', 80);

            //Branco - Posição: 261 a 494 - Picture: X()
            $linha.= str_repeat(' ', 234);

            //Nº seq. do registro (O arq. Header sempre será "000001').
            //Posição: 495 a 500 - Picture: 9(6)
            $linha.= str_pad(++$this->nrseqreg, 6, '0', STR_PAD_LEFT);

            return $linha."\r\n";

        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @param Boletpagar $boleto
     * @param bool $autorizaAgendamento
     * @throws \Exception
     * @return string
     */
    public function geraRegistroTransacao(Boletpagar $boleto, $autorizaAgendamento =  false){

        try {
            ++$this->nrseqreg;
            $fator = '';
            $cdagencia = '';
            $cdconta = '';

            //Busca dados do Código de Barras ou da linah digitável
            if ($boleto->getCdbarrapag()) {
                $cdbanco = str_pad(substr($boleto->getCdbarrapag(), 0, 3), 3, '0', STR_PAD_LEFT);
                $cdmoeda = str_pad(substr($boleto->getCdbarrapag(), 3, 1), 1, '0', STR_PAD_LEFT);
                $cddigito = str_pad(substr($boleto->getCdbarrapag(), 4, 1), 1, '0', STR_PAD_LEFT);
                $fator = str_pad(substr($boleto->getCdbarrapag(), 5, 4), 4, '0', STR_PAD_LEFT);
                $valor = str_pad(substr($boleto->getCdbarrapag(), 9, 10), 10, '0', STR_PAD_LEFT);
                $cdagencia = str_pad(substr($boleto->getCdbarrapag(), 19, 4), 4, '0', STR_PAD_LEFT);
                $cdconta = str_pad(substr($boleto->getCdbarrapag(), 36, 7), 7, '0', STR_PAD_LEFT);
                $cdcampo = str_pad(substr($boleto->getCdbarrapag(), 19, 25), 25, '0', STR_PAD_LEFT);
            } else {
                $linhaBarra = '';
                $dslinhbarpag = $boleto->getDslinhbarpag();
                for ($i = 0; $i < strlen($dslinhbarpag); $i++) {
                    if (in_array($dslinhbarpag[$i], array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9'))) {
                        $linhaBarra .= $dslinhbarpag[$i];
                    }
                }
                $cdbanco = str_pad(substr($linhaBarra, 0, 3), 3, '0', STR_PAD_LEFT);
                $cdmoeda = str_pad(substr($linhaBarra, 3, 1), 1, '0', STR_PAD_LEFT);
                $cddigito = str_pad(substr($linhaBarra, 32, 1), 1, '0', STR_PAD_LEFT);
                $valor = str_pad(substr($linhaBarra, 33, 14), 14, '0', STR_PAD_LEFT);
                $cdcampo = str_pad(substr($linhaBarra, 4, 25), 25, '0', STR_PAD_LEFT);
            }

            //Tipo de registro = registro Transação - Posição: 1 a 1 - Picture: 9(1)
            $linha = self::ID_REGISTRO_TRANSACAO;

            ///Tipo de inscrição do fornecedor (1 = CPF/ 2 = CGC/ 3 = Outros) -
            //Posição: 2 a 2 - Picture: 9(1)
            if ($boleto->getIdtpicnabbop()) {
                if ($boleto->getIdtpicnabbop() == Empresa::TIPO_INSCRICAO_FISICA) {
                    $linha .= self::TIPO_INS_JUR_CPF;
                } elseif ($boleto->getIdtpicnabbop() == Empresa::TIPO_INSCRICAO_CGC) {
                    $linha .= self::TIPO_INS_JUR_CNPJ;
                } else {
                    $linha .= self::TIPO_INS_JUR_OUTROS;
                }

            } else {
                if ($boleto->getForneced()->getIdtpijurforn() == Empresa::TIPO_INSCRICAO_FISICA) {
                    $linha .= self::TIPO_INS_JUR_CPF;
                } elseif ($boleto->getForneced()->getIdtpijurforn() == Empresa::TIPO_INSCRICAO_CGC) {
                    $linha .= self::TIPO_INS_JUR_CNPJ;
                } else {
                    $linha .= self::TIPO_INS_JUR_OUTROS;
                }
            }

            //(CGC/CPF do fornecedor) + (CGC <-> Filial) + (CGC/CPF <-> Controle).
            //(15 posições numericas - 3 a 17
            if ($boleto->getIdtpicnabbop() == Empresa::TIPO_INSCRICAO_FISICA) {
                if ($boleto->getNrinscnabbop()) {
                    $linha .= str_pad(substr(trim(Util::removeMascaraInscricao($boleto->getNrinscnabbop())), 0, 9) .
                        str_repeat('0', 4) .
                        substr(trim(Util::removeMascaraInscricao($boleto->getNrinscnabbop())), 9, 2), 15, '0', STR_PAD_LEFT);
                } else {
                    $linha .= str_pad(substr(trim(Util::removeMascaraInscricao(
                            $boleto->getForneced()->getNrinsjurforn())), 0, 9) .
                            str_repeat('0', 4) .
                            substr(trim(Util::removeMascaraInscricao($boleto->getForneced()->getNrinsjurforn())), 9, 2)
                        , 15, '0', STR_PAD_LEFT);
                }
            } else { // Se for Inscrição Jurídica ou Outros pega os 15 primeiros dígitos.
                if ($boleto->getNrinscnabbop()) {
                    $linha .= str_pad(substr(trim(Util::removeMascaraInscricao($boleto->getNrinscnabbop())), 0, 15), 15, '0',
                        STR_PAD_LEFT);

                } else {
                    $linha .= str_pad(substr(trim(Util::removeMascaraInscricao($boleto->getForneced()->getNrinsjurforn()))
                        , 0, 15), 15, '0', STR_PAD_LEFT);
                }
            }

            //Nome do fornecedor - Posição: 18 a 47 - Picture: X(30)
            $linha .= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(), 0, 30)), 30, ' ',
                STR_PAD_RIGHT);

            //Endereço do fornecedor - Posição: 48 a 87 - Picture: X(40)
            $endforn = $boleto->getForneced()->getEndefornecobr();
            if (!$endforn)
                $endforn = $boleto->getForneced()->getEndeforneprinc();
            $linha .= str_pad(substr(Util::campoCnabValido($endforn->getDsendeforn()), 0, 40), 40, ' ', STR_PAD_RIGHT);

            //CEP do Fornecedor + complemento do CEP -
            //Posição: 88 a 95 - Picture: X(5) +
            //Posição: 93 a 95 - Picture: X(3)
            $linha .= str_pad(Util::campoCnabValido(trim(substr($endforn->getNrcepforn(),0,5).
                                                                substr($endforn->getNrcepforn(),-3))),
                              8, '0',STR_PAD_RIGHT);

            //Código do banco do fornecedor - Posição: 96 a 98 - Picture: 9(3)

            if ($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG) {
                $cdbancoforn = self::CODIGO_COMPENSACAO_BRADESCO;
            } else {
                $cdbancoforn = self::getBanco($boleto, $boleto->getTpbaixapag()->getIdbaixautilp());
            }
            $linha .= str_pad($cdbancoforn, '0', STR_PAD_LEFT);

            //Busca a modalidade do pagamento
            $modalidade = self::getModalidadePagamento($boleto, $cdbancoforn);


            if ($modalidade == self::MODALIDADE_BOLETO_BANCARIO and $cdbancoforn != self::CODIGO_COMPENSACAO_BRADESCO) {
                // Para modalida 31 e banco diferente de 237 a Agência e Conta Corrente
                // serão buscados do campo Informações Complementares. Por isso deve ser informado zeros.
                $linha .= str_repeat('0', 21);
            } else
            {
                if ($modalidade == self::MODALIDADE_BOLETO_BANCARIO and $cdbancoforn == self::CODIGO_COMPENSACAO_BRADESCO) {
                    $linha .= str_pad($cdagencia, 5, '0', STR_PAD_LEFT) . Util::mod11_b7($cdagencia);
                    $linha .= str_pad($cdconta, 13, '0', STR_PAD_LEFT) . Util::mod11_b7($cdconta) . ' ';
                } else
                {
                    if ($boleto->getTpbaixapag()->getIdbaixautilp() ==
                        Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG
                    ) {

                        //Código da agência do fornecedor - Posição: 99 a 103 - Picture: 9(5)
                        $linha .= str_pad(substr(trim($this->contcorr->getCdagencia()), 0, -1), 5, '0', STR_PAD_LEFT);

                        //Digito da agência do fornecedor - Posição: 104 a 104 - Picture: 9(1)
                        $linha .= substr(trim($this->contcorr->getCdagencia()), -1);

                        //Reservado  - Posição: 105 a 119 - Picture: X(15)
                        $linha .= str_repeat(' ', 15);

                    } else {

                        //Código da agência do fornecedor - Posição:9 99 a 103 - Picture: 9(5)
                        //MAIS
                        //Digito da agência do fornecedor - Posição: 104 a 104 - Picture: 9(1)

                        //Busca a máscara da CONTA e AGÊNCIA do fornecedor cadastrada na tela de Banco
                        /** @var $bancoFornecedor Banco */
                        $bancoFornecedor = self::getBancoForn($boleto->getCdbancbolp(), $boleto->getNrorg());

                        //{========= Se o Banco do fornecedor estiver cadastrado no sistema e parametrizado com as máscaras,
                        // entrará nesse if =========}
                        if ($bancoFornecedor and $bancoFornecedor->getCdpictagen()) {
                            //{ se na máscara da agência NÃO tem '-' }
                            if (strpos($bancoFornecedor->getCdpictagen(), '-') === false) {
                                //Agencia
                                $linha .= str_pad(substr(trim(Util::campoCnabValido(self::getAgencia($boleto))), 0, 5)
                                    , 5, '0', STR_PAD_LEFT);
                                //DV
                                $linha .= ' ';
                            } else {
                                //{ Caso a máscara da agência TENHA '-' }
                                //Agencia
                                $linha .= str_pad(substr(trim(Util::campoCnabValido(self::getAgencia($boleto))), 0,
                                        strpos($bancoFornecedor->getCdpictagen(), '-'))
                                    , 5, '0', STR_PAD_LEFT);

                                //Se foi cadastrada a agencia do favorecido como 9999 na tela de boleto e
                                //tempos depois foi modificada a máscara para 9999-9 (e o campo anterior não foi atualizado),
                                //qnd for pegar a parte do -9, nao tem e ele fica vazio, este if arruma este problema.
                                if (strlen(substr(trim(Util::campoCnabValido(self::getAgencia($boleto)))
                                    , strpos($bancoFornecedor->getCdpictagen(), '-'))) > 0
                                ) {
                                    //DV
                                    $pos = strpos($bancoFornecedor->getCdpictagen(), '-') + 1;
                                    $linha .= substr(trim(Util::campoCnabValido(self::getAgencia($boleto)))
                                        , $pos, 1);
                                } else {
                                    $linha .= ' ';
                                }
                            }
                        } else {
                            //{======== Se o Banco do fornecedor NÃO estiver cadastrado no sistema,
                            //NÃO parametrizado com as máscaras, entrará nesse else ====}
                            if (strpos($boleto->getCdagenbolp(), '-') === false) {
                                // { se o que foi digitado no campo de agencia do favorecido da tela de boleto NÃO tem '-' }
                                //Agencia
                                $linha .= str_pad(substr(trim(self::getAgencia($boleto)), 0, 5), 5, '0', STR_PAD_LEFT);

                                //DV
                                $linha .= ' ';
                            } else {
                                //{ caso o que foi digitado no campo de agencia do favorecido da tela de boleto TENHA '-' }
                                //Agencia
                                $linha .= str_pad(Util::campoCnabValido(substr(self::getAgencia($boleto), 0,
                                    strpos($boleto->getCdagenbolp(), '-')
                                )), 5, '0', STR_PAD_LEFT);

                                //DV
                                $pos = strpos($boleto->getCdagenbolp(), '-') + 1;
                                $linha .= substr(trim(Util::campoCnabValido(self::getAgencia($boleto)))
                                    , $pos, 1);
                            }
                        }

                        //Conta corrente do fornecedor - Posição: 105 a 117 - Picture: 9(13)
                        //MAIS
                        //Digito da C/C do fornecedor - Posição: 118 a 119 - Picture: 9(2)
                        //Obs: A 2 últimas posição da C/C é res. para o Dig. Verif.

                        //{========== Se o Banco do fornecedor estiver cadastrado no sistema e parametrizado
                        //com as máscaras, entrará nesse if ===========}
                        if ($bancoFornecedor and $bancoFornecedor->getCdpictcont()) {
                            //{ se na máscara da conta corrente TEM '-' }
                            if (strpos($bancoFornecedor->getCdpictcont(), '-')) {

                                //CC
                                $linha .= str_pad(Util::campoCnabValido(substr(self::getConta($boleto), 0,
                                        strpos($bancoFornecedor->getCdpictcont(), '-'))),
                                    13, '0', STR_PAD_LEFT);

                                //Se foi cadastrada a cc do favorecido como 9999 na tela de boleto e tempos depois
                                //foi modificada a máscara para 9999-9 (e o campo anterior não foi atualizado),
                                //qnd for pegar a parte do -9, nao tem e ele fica vazio, este if arruma este problema
                                if (strlen(substr(trim(Util::campoCnabValido(self::getConta($boleto)))
                                    , strpos($bancoFornecedor->getCdpictcont(), '-'))) > 0
                                ) {
                                    //DV
                                    $pos = strpos($bancoFornecedor->getCdpictcont(), '-') + 1;
                                    $linha .= substr(trim(Util::campoCnabValido(self::getConta($boleto)))
                                        , $pos, 1);
                                } else {
                                    $linha .= str_repeat(' ', 2);
                                }
                            } else {
                                //se na máscara da conta corrente NÃO tem '-' }
                                //CC
                                $linha .= str_pad(Util::campoCnabValido(substr(self::getConta($boleto), 0, 13)
                                ), 13, '0', STR_PAD_LEFT);

                                //DV
                                $linha .= str_repeat(' ', 2);
                            }
                        } // {======== Se o Banco do fornecedor NÃO estiver cadastrado no sistema, NÃO parametrizado com
                        //as máscaras, entrará nesse else ====}
                        else {
                            if (strpos($boleto->getCdccorbolp(), '-')) {
                                //CC
                                $linha .= str_pad(Util::campoCnabValido(substr(self::getConta($boleto), 0,
                                        strpos($boleto->getCdccorbolp(), '-'))
                                ), 13, '0', STR_PAD_LEFT);

                                //DV
                                $linha .= Util::campoCnabValido(substr($boleto->getCdccorbolp(),
                                    strpos($boleto->getCdccorbolp(), '-') + 1));
                            } else {

                                //CC
                                $linha .= str_pad(Util::campoCnabValido(substr($boleto->getCdccorbolp(), 0,
                                        13)
                                ), 13, '0', STR_PAD_LEFT);

                                //DV
                                $linha .= ' ';

                            }
                        }
                    }
                }
            }
                //Número do pagamento - Posição: 120 a 135 - Picture: X(16)
                $linha .= str_pad(trim($boleto->getCdfilial() . $boleto->getNrlanbolpag()), 16, ' ', STR_PAD_RIGHT);

                if ($modalidade == self::MODALIDADE_BOLETO_BANCARIO and
                    $cdbancoforn == self::CODIGO_COMPENSACAO_BRADESCO
                ) {
                    //Carteira - Posição: 136 a 138 - Picture: 9(3)
                    $linha .= str_pad(substr($cdcampo, 4, 2), 3, '0', STR_PAD_LEFT);

                    //Nosso Número - Posição: 139 a 150 - Picture: 9(012)
                    $linha .= str_pad(substr($cdcampo, 6, 11), 12, '0', STR_PAD_LEFT);
                } else {
                    //Carteira - Posição: 136 a 138 - Picture: 9(3)
                    $linha .= str_repeat('0', 3);

                    //Nosso Número - Posição: 139 a 150 - Picture: 9(012)
                    $linha .= str_repeat('0', 12);
                }

                //Seu Número - Posição: 151 a 165 - Picture: X(15)
                $linha .= str_repeat(' ', 15);

                //Data de vencimento - Posição: 166 a 173 - Picture: 9(8)
                $linha .= $boleto->getDtvenbolpag()->format('Ymd');

                //Data de emissão do documento - Posição: 174 a 181 - Picture: 9(8)
                $linha .= str_repeat('0', 8);

                //Data de limite para desconto - Posição: 182 a 189 - Picture: 9(8)
                if ($boleto->getTpbaixapag()->getIdbaixautilp() ==
                    Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO
                ) {
                    // O valor possui duas casas decimais mas não é separado por virgula,
                    // por isso que está sendo dividido por Cem.
                    $valorDocumento = round(floatval($valor) / 100, 2);
                } else {
                    /** @var $rep rep_Contapagar */
                    $rep = Contapagar::getRepository();
                    $valorDocumento = $rep->getVrdocumento($boleto->getCdfilial(),
                                                           $boleto->getNrlanbolpag(),
                                                           $boleto->getNrorg());

                    $valorDocumento = round(floatval($valorDocumento), 2);

                }
                $this->valorTotalPgto += $boleto->getVrfinalbpag();

                $vrAcrescimoDesconto = round($boleto->getVrfinalbpag() - $valorDocumento, 2);

                if ($vrAcrescimoDesconto < 0) {
                    $linha .= $boleto->getDtvenbolpag()->format('Ymd');
                } else {
                    $linha .= str_repeat('0', 8);
                }

                //Valor do documento - Posição: 190 a 190 - Fixo zero
                $linha .= '0';

                //Fator de Vencimento - Posição: 191 a 194
                if ($modalidade == self::MODALIDADE_BOLETO_BANCARIO) {
                    $linha .= $fator;
                } else {
                    $linha .= str_repeat('0', 4);
                }

                //Valor do documento - Posição: 195 a 204
                $linha .= str_pad(str_replace(array('.', ','), '', $valorDocumento), 10, '0', STR_PAD_LEFT);

                //Valor do pagto - Posição: 205 a 219 - Picture: 9(13)V9(2)
                $linha .= str_pad(str_replace(array('.', ','), '',
                    number_format($boleto->getVrfinalbpag(), 2)
                ), 15, '0', STR_PAD_LEFT);

                if ($valorDocumento != 0) {

                    if ($vrAcrescimoDesconto < 0) {
                        //Valor do desconto - Posição: 220 a 234 - Picture: 9(13)V9(2)
                        $linha .= str_pad(str_replace(array('.', ','), '', $vrAcrescimoDesconto * -1), 15, '0', STR_PAD_LEFT);

                        //Valor do Acrescimo - Posição: 235 a 249 - Picture: 9(13)V9(2)
                        $linha .= str_repeat('0', 15);
                    } else {
                        if ($vrAcrescimoDesconto > 0) {
                            //Valor do desconto - Posição: 220 a 234 - Picture: 9(13)V9(2)
                            $linha .= str_repeat('0', 15);

                            //Valor do Acrescimo - Posição: 235 a 249 - Picture: 9(13)V9(2)
                            $linha .= str_pad(str_replace(array('.', ','), '', $vrAcrescimoDesconto * -1), 15, '0', STR_PAD_LEFT);
                        } else {
                            //Valor do desconto + Valor do Acrescimo - Posição: 220 a 249
                            $linha .= str_repeat('0', 30);

                        }
                    }

                } else {
                    //Valor do desconto + Valor do Acrescimo - Posição: 220 a 249
                    $linha .= str_repeat('0', 30);
                }

                //Tipo de documento (01 - Nota Fiscal/Fatura
                //                   02 - Fatura
                //                   03 - Nota Fiscal
                //                   04 - Duplicata
                //                   05 - Outros)
                //Posição: 250 a 251 - Picture: 9(2)
                $linha .= '04';


                //Número nota Fiscal/Fatura/Duplicata - Posição: 252 a 261 - Picture: 9(10)
                /** @var $contapagar Contapagar */
                $contapagar = self::getContaPagar($boleto->getNrlanbolpag(),
                    $boleto->getCdfilial(),
                    $boleto->getNrorg());
                if ($contapagar and $contapagar->getLanctonf()->getNrnotafisc()) {
                    $linha .= str_pad(substr(Util::campoCnabValido(trim($contapagar->getLanctonf()->getNrnotafisc()))
                        , 0, 10), 10, ' ', STR_PAD_RIGHT);

                    //Série documento - Posição: 262 a 263 - Picture: X(2)
                    $linha .= str_pad(substr(Util::campoCnabValido(trim($contapagar->getLanctonf()->getCdserieentr()))
                        , 0, 2), 2, ' ', STR_PAD_RIGHT);
                } else {
                    $linha .= str_repeat('0', 10);
                    $linha .= str_repeat(' ', 2);

                }

                //Modalidade de pagamento - Crédito em C/C - Cheque OP -
                //Doc - Titulo Bradesco - Títulos terceiros
                //Posição: 264 a 265 - Picture: 9(2)
                $linha .= $modalidade;

                //Data para efetivação do pagamento - Posição: 266 a 273 - Picture: 9(8)
                $linha .= $boleto->getDtvenbolpag()->format('Ymd');

                //Moeda (Código CNAB) - Posição: 274 a 276 - Picture: X(3)
                $linha .= str_repeat(' ', 3);

                //Situação do agendamento - Posição: 277 a 278 - Picture: X(2)
                $linha .= '01';

                //Informação de retorno 1 - Posição: 279 a 280 - Picture: X(2)
                $linha .= str_repeat(' ', 2);

                //Informação de retorno 2 - Posição: 281 a 282 - Picture: X(2)
                $linha .= str_repeat(' ', 2);

                //Informação de retorno 3 - Posição: 283 a 284 - Picture: X(2)
                $linha .= str_repeat(' ', 2);

                //Informação de retorno 4 - Posição: 285 a 286 - Picture: X(2)
                $linha .= str_repeat(' ', 2);

                //Informação de retorno 5 - Posição: 287 a 288 - Picture: X(2)
                $linha .= str_repeat(' ', 2);

                //Tipo de movimento (0 = inclusão/ 5 = alteração/ 9 = exclusão)
                //Posição: 289 a 289 - Picture: 9(1)
                if ($this->remessa->getIdtipcnabpag() == self::EXCLUSAO) {
                    $linha .= '9';
                    //Código do movimento (00 = Autor. agendamento / 25 = Desaut. agendamento)
                    //Posição: 290 a 291
                    $linha .= ' ';

                } else {
                    $linha .= '0';
                    //Código do movimento (00 = Autor. agendamento / 25 = Desaut. agendamento)
                    //Posição: 290 a 291

                    $linha .= $autorizaAgendamento ? '00' : '25';
                }

                //Endereço do sacado (Fixo Brancos) - Posição: 292 a 331 - Picture: X(40)
                $linha .= str_repeat(' ', 40);

                //Sacador avalista - Posição: 332 a 371 - Picture: X(40)
                $linha .= str_repeat(' ', 40);

                //Reserva - Posição: 372 a 372 - Picture: X(1)
                $linha .= ' ';

                //Nível da informação de Retorno - Posição: 373 a 373 - Picture: X(1)
                $linha .= ' ';

                //********************************************************//
                // Informações complementares - Posição: 374 a 413        //
                // Picture: X(40)                                         //
                //********************************************************//

                switch ($modalidade) {

                    //Credito em conta corrente
                    case self::MODALIDADE_CREDITO_EM_CONTA:
                        $linha .= str_repeat(' ', 40);
                        break;

                    case self::MODALIDADE_DOC:
                        //Doc - Posição 374 a 374 -- Tipo C ou D
                        $linha .= 'C';
                        //Doc - Posição 375 a 380 -- Numero do Doc
                        $linha .= str_repeat('0', 6);
                        //Doc - Posição 381 a 382 -- Codigo da Finalidade
                        $linha .= '01';
                        //Tipo de Conta - Posição 383 a 384
                        $linha .= '01';
                        //Brancos - Posição 385 a 413
                        $linha .= str_repeat(' ', 29);
                        break;

                    case self::MODALIDADE_TED_CIP:
                    case self::MODALIDADE_TED_STR:
                        //Doc - Posição 374 a 374 -- Tipo C ou D
                        $linha .= 'C';
                        //Doc - Posição 375 a 380 -- Numero do Doc
                        $linha .= str_repeat('0', 6);
                        //Doc - Posição 381 a 382 -- Codigo da Finalidade
                        $linha .= '01';
                        //Tipo de Conta - Posição 383 a 384
                        $linha .= '01';
                        //Brancos - Posição 385 a 413
                        $linha .= str_repeat(' ', 29);
                        break;

                    case self::MODALIDADE_BOLETO_BANCARIO:
                        //Campo Livre - posição 374 a 398
                        $linha.= str_pad(substr($cdcampo, 0, 25), 25, '0', STR_PAD_LEFT);
                        //Dígito do código de barras - posição 399 a 399
                        $linha.= substr($cddigito,0,1);
                        //Código da Moeda - posição 400 a 400
                        $linha.= substr($cdmoeda,0,1);
                        //Brancos - posição 401 a 413
                        $linha.= str_repeat(' ',13);
                        break;

                    default:
                        $linha.= str_repeat(' ',40);
                }

                //Código de área na empresa - Posição: 414 a 415 - Picture: 9(2)
                $linha.= '00';

                //Campo para uso da empresa - Posição: 416 a 450 - Picture: X(35)
                $linha.= str_pad(trim($boleto->getCdfilial().$boleto->getNrlanbolpag()),35,' ',STR_PAD_RIGHT);

                //Reserva - Posição: 451 a 472 - Picture: X(22)
                $linha.= str_repeat(' ',22);

                //Código de lançamento - Posição: 473 a 477 - Picture: 9(5)
                $linha.= '00000';

                //Reserva - Posição: 478 a 478 - Picture: X(1)
                $linha.= ' ';

                //Tipo de conta do fornecedor - Posição: 479 a 479 - Picture: 9(1)
                $linha.= '1';
                
                $contcorr = Contcorr::findOneByNrorgAndCdreducont($this->remessa->getNrorg(), $this->remessa->getCdreducont());
                
                //Conta complementar - Posição: 480 a 486 - Picture: 9(7)
                $linha.= str_pad(trim(substr(Util::campoCnabValido($contcorr->getCdcedenteban()),0,7)),7,'0',STR_PAD_LEFT);

                //Reserva - Posição: 487 a 494 - Picture: X(8)
                $linha.= str_repeat(' ',8);

                //Nº sequencial do registro - Posição: 495 a 500 - Picture: 9(6)
                $linha.= str_pad($this->nrseqreg,6,'0',STR_PAD_LEFT);

                return $linha."\r\n";

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Boletpagar $boleto
     * @param $cdbancoforn
     * @return string
     */
    private function getModalidadePagamento(Boletpagar $boleto, $cdbancoforn){
        $idbaixa = $boleto->getTpbaixapag()->getIdbaixautilp();
        if($idbaixa == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO)
            return self::MODALIDADE_BOLETO_BANCARIO;

        else{
            if($idbaixa == Tipobaixapag::TIPO_BAIXA_CNAB_ORDEM_PAG){
                return self::MODALIDADE_ORDEM_PAGAMENTO;
            }else{
                if($cdbancoforn == self::CODIGO_COMPENSACAO_BRADESCO ){
                    return self::MODALIDADE_CREDITO_EM_CONTA;
                }else{
                    if($boleto->getVrfinalbpag() >=5000){
                        if($idbaixa == Tipobaixapag::TIPOBAIXA_TED_CIP)
                            return self::MODALIDADE_TED_CIP;
                        else{
                            if($idbaixa == Tipobaixapag::TIPO_BAIXA_TED_STR)
                                return self::MODALIDADE_TED_STR;
                        }
                    }
                    else{
                        return self::MODALIDADE_DOC;
                    }
                }
            }
        }
    }


    /**
     * @param $cdbanco
     * @param \Fiscal\Entities\Boletpagar $boleto
     * @internal param $formaLancamento
     * @return mixed
     */
    public function geraHeaderLote($cdbanco, $boleto =  null)
    {
        // TODO: Implement geraHeaderLote() method.
    }


    public function geraTraillerLote($somavalores = null)
    {
        // TODO: Implement geraTraillerLote() method.
    }

    /**
     * @return string
     */
    public function geraTrailerArquivo()
    {
        $this->nrseqreg++;

        //Tipo de registro = registro trailer de arquivo - Posição: 1 a 1 - Picture: 9(1)
        $linha = self::ID_REGISTRO_TRAILLER;

        //Total de Qtde de registro do arquivo - Posição: 2 a 7 - Picture: 9(6)
        $linha.= str_pad($this->nrseqreg,6,'0',STR_PAD_LEFT);

        //Total dos valores de pagamentos - Posição: 8 a 24 - Picture: 9(17)
        $linha.=  str_pad(str_replace(array('.',','),'',
                          number_format($this->valorTotalPgto,2)
                          ),17,'0',STR_PAD_LEFT);

        //Reserva - Posição: 25 a 494 - Picture: X(470)
        $linha.= str_repeat(' ',470);

        //Número sequencial no arquivo - Posição: 495 a 500 - Picture: 9(6)
        $linha.= str_pad($this->nrseqreg,6,'0',STR_PAD_LEFT);

        return $linha."\r\n";
    }

    /**
     * @param $caminhoArquivo
     * @param \Financeiro\Entities\Cnabpag $boletos
     * @param bool $autorizaAgendamento
     * @throws \Exception
     * @internal param $remessa
     * @return mixed
     */
    public function geraArquivo($caminhoArquivo, $boletos, $autorizaAgendamento = false, $idopecnabpag = null)
    {
        try{
            $file = fopen($caminhoArquivo.'.txt', "a+");
            $headerarquivo = self::geraHeaderArquivo();
            fprintf($file, "%s",$headerarquivo);
            foreach($boletos as $boleto){

                $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag($boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']);

                $registro = self::geraRegistroTransacao($boleto,$autorizaAgendamento);
                fprintf($file, "%s",$registro);
            }
            $trailler = self::geraTrailerArquivo();
            fprintf($file, "%s",$trailler);
            fclose($file);
        }
        catch (\Exception $e){
            fclose($file);
            throw $e;
        }
    }

    /**
     * @param Boletpagar $boleto
     * @internal param null $idbaixautilp
     * @return null|string
     */
    private function getConta(Boletpagar $boleto){

        if ($boleto->getBaixapag()->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
            if($boleto->getCdbarrapag()){
                return substr($boleto->getCdbarrapag(),36,7).'-'. Util::mod11_b7(36,7);
            }
        }else{
            return $boleto->getCdccorbolp();
        }
        return null;
    }

    /**
     * @param Boletpagar $boleto
     * @internal param null $idbaixautilp
     * @return null|string
     */
    private function getAgencia(Boletpagar $boleto){
        if ($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
            if($boleto->getCdbarrapag()){
                return substr($boleto->getCdbarrapag(),19,4). Util::mod11_b7(19,4);
            }
        }else{
            return $boleto->getCdagenbolp();
        }
        return null;
    }

    /**
     * @param $nrlanbolpag
     * @param $cdfilibpag
     * @param $nrorg
     * @return Contapagar
     */
    private function getContaPagar($nrlanbolpag, $cdfilibpag, $nrorg){
        /** @var $contaPagar Contapagar */
        $contaPagar = Contapagar::findOneByNrlanbolpagAndCdfilibpagAndNrorg($nrlanbolpag, $cdfilibpag, $nrorg);
        return $contaPagar;
    }
}