<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco;

use Teknisa\Libs\Util\DateUtil;
use Geral\Entities\Banco;
use Geral\Util\NovoCodigo;
use Geral\Entities\Contcorr;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\GeraEmissaoBoletoService;

/**
 * Class GeraEmissaoBoletoSantanderService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco
 */
class GeraEmissaoBoletoSantanderService extends GeraEmissaoBoletoService
{

    const TAMANHO_NOSSO_NUMERO                   = 12;
    const MODULO_DIGITO_VERIFICADOR_NOSSONUM     = 11;
    const MODULO_DIGITO_VERIFICADOR_LINHADIGIT   = 10;
    const FATOR_VENCIMENTO                       = '07/10/1997';


    public function getNossoNumeroComDigito()
    {
        $codigioReduzido = $this->getCodigoReduzido();
        $cdcontador      = self::PREFIXO_CDCONTADOR.$codigioReduzido;
        $nossoNumero     = NovoCodigo::geraCodigo (
            $this->boleto->getNrorg(), $cdcontador, 1, self::TAMANHO_NOSSO_NUMERO
        );
        return $nossoNumero.$this->getDigitoVerificador($nossoNumero, self::SEQUENCIA_NOSSO_NUMERO);
    }

    /** Função que monta a Linha Digitável do boleto de acordo com o Layout do Banco Santander.
     * @param $nossoNumero
     * @return string
     */
    public function getLinhaDigitavel($nossoNumero)
    {
        $grupo1  = $this->getIdentificadorBanco().self::MOEDA_REAL."9".$this->getCodigoCedente();
        $grupo1 .= $this->getVerificadorGruposLinhaDigitavel($grupo1);

        $grupo2  = str_pad(substr($this->getCdcedenteban(),4,3),3,"0",STR_PAD_LEFT);
        $grupo2 .= substr($nossoNumero,0,7);
        $grupo2 .= substr($this->getVerificadorGruposLinhaDigitavel($grupo2),0,1);

        $grupo3 = substr($nossoNumero,7,6);
        $grupo3 .= '0';
        $grupo3 .= str_pad($this->boleto->getCdcartcobbol(), 3, "0", STR_PAD_LEFT);
        $grupo3 .= substr($this->getVerificadorGruposLinhaDigitavel($grupo3),0,1);

        $grupo4 = $this->getDigitoCodigoBarras();

        $grupo5 = $this->getFatorVencimento();
        $grupo5 .= $this->getValorNominal();
        $grupo5 = str_pad($grupo5,"0",16);

        return self::concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5);
    }

    public function getCodigoDeBarras($nossoNumero)
    {
        $codigoDeBarras  = $this->getIdentificadorBanco();
        $codigoDeBarras .= self::MOEDA_REAL;
        //Esta string será substituída pelo código do dígito verificador
        $codigoDeBarras .= "_SUBSTITUICAO_DV_";
        $codigoDeBarras .= $this->getFatorVencimento();
        $codigoDeBarras .= $this->getValorNominal();
        $codigoDeBarras .= "9";
        $codigoDeBarras .= $this->getCodigoCedenteCodBar();
        $codigoDeBarras .= substr($nossoNumero, 0, 13);
        $codigoDeBarras .= "0";
        $codigoDeBarras .= str_pad($this->boleto->getCdcartcobbol(), 3, "0", STR_PAD_LEFT);

        $digito         = $this->getDigitoVerificador(str_replace("_SUBSTITUICAO_DV_", "", $codigoDeBarras));
        $codigoDeBarras = str_replace("_SUBSTITUICAO_DV_", $digito, $codigoDeBarras);

        return $codigoDeBarras;
    }

    private function getValorNominal()
    {
        $valorBoleto = (string)$this->getValorBoleto();
        $valorBoleto = str_replace(',','.',$valorBoleto);
        $valorBoleto = number_format($valorBoleto, 2, '.', '');
        $arrayExclusao = array('.', ',');
        $valorBoleto = str_replace($arrayExclusao, "", $valorBoleto);
        $valorBoleto = str_pad($valorBoleto, 10, "0", STR_PAD_LEFT);
        return $valorBoleto;
    }


    private function getCodigoCedente()
    {
        if ($this->getCdcedenteban() == false) {
            $codigoCedente = '0000';
        } else {
            $codigoCedente = str_pad(substr($this->getCdcedenteban(), 0, 4),4,"0", STR_PAD_LEFT);
        }

        return $codigoCedente;
    }
    
    private function getCodigoCedenteCodBar()
    {
        if ($this->getCdcedenteban() == false) {
            $codigoCedente = '00000000';
        } else {
            $codigoCedente = str_pad(substr($this->getCdcedenteban(), 0, 7),7,"0", STR_PAD_LEFT);
        }

        return $codigoCedente;
    }

    private function getIdentificadorBanco()
    {
        $identificadorBanco = $this->getCdbanccnab();
        return $identificadorBanco;
    }

    private function getFatorVencimento()
    {
        $datavenc            = DateUtil::truncateData($this->getDataVencimento());
        $dataFatorVencimento = DateUtil::truncateData(DateUtil::getDataDeString(self::FATOR_VENCIMENTO));
        $fatorDeVencimento   = DateUtil::getQtdeDiasEntre($dataFatorVencimento, $datavenc);
        $fatorDeVencimento   = str_pad($fatorDeVencimento, 4, "0", STR_PAD_LEFT);
        return $fatorDeVencimento;
    }

    private function getVerificadorGruposLinhaDigitavel($linhaDigitavel)
    {
        $linhaDigitavel = strrev($linhaDigitavel);
        $peso = 2;
        $total = 0;
        for($i = 0; $i<=strlen($linhaDigitavel);$i++){
            $result = (int) substr($linhaDigitavel,$i,1)* $peso;
            if ($result > 9){
                $total += (int)substr((string)$result, 0, 1) + (int)substr((string)$result, 1,1);
            }else{
                $total += ((int) substr($linhaDigitavel,$i,1)* $peso);
            }
            $peso = $this->incrementaValorPeso($peso, 1, 2 );

       }
       $digito = $total % 10;

        if($digito == 0)
            return $digito;
        return 10 - $digito;
    }

    public function getRelatorioBoleto($nrorg, $cdreducont)
	{
        /** @var $contcorr Contcorr */
        $contcorr = Contcorr::findOneByNrorgAndCdreducont($nrorg,$cdreducont);
        $contcorr->getcdbanco();

        /** @var $dsarqbolcomp Banco*/
        $dsarqbolcomp = Banco::findOneByCdbanco($contcorr->getcdbanco());
        $string = $dsarqbolcomp->getDsarqbolcomp();

        $tam = strlen($string);
        $tam_b = strrpos($string,'\\');
        $ext = -($tam - strrpos($string,'.'));

        if($tam_b === false)
            return NULL;

        else return ($string_p = substr($string, ++$tam_b, $ext));
    }

    private function concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5)
    {
       	$linhadigitavel = substr($grupo1,0,5).'.'.substr($grupo1, 5).' ';

       	$linhadigitavel .= substr($grupo2,0, 5).'.'.substr($grupo2, 5).' ';

       	$linhadigitavel .= substr($grupo3,0, 5).'.'.substr($grupo3, 5).' ';

       	$linhadigitavel .= $grupo4.' ';

       	$linhadigitavel .= $grupo5;

       	return $linhadigitavel;
    }

    public function chamaRelatorio()
    {

    }
}
